<?php

namespace Drupal\box\Entity;

use Drupal\box\BoxTypeListBuilder;
use Drupal\box\Form\BoxTypeDeleteForm;
use Drupal\box\Form\BoxTypeForm;
use Drupal\Core\Config\Entity\ConfigEntityBundleBase;
use Drupal\Core\Entity\Attribute\ConfigEntityType;
use Drupal\Core\Entity\Routing\AdminHtmlRouteProvider;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\user\Entity\EntityPermissionsRouteProvider;

/**
 * Defines the Box bundle entity.
 */
#[ConfigEntityType(
  id: 'box_type',
  label: new TranslatableMarkup('Box type'),
  label_collection: new TranslatableMarkup('Box types'),
  label_singular: new TranslatableMarkup('box type'),
  label_plural: new TranslatableMarkup('box types'),
  config_prefix: 'box_type',
  entity_keys: [
    'id' => 'id',
    'label' => 'label',
  ],
  handlers: [
    'list_builder' => BoxTypeListBuilder::class,
    'form' => [
      'add' => BoxTypeForm::class,
      'edit' => BoxTypeForm::class,
      'delete' => BoxTypeDeleteForm::class,
    ],
    'route_provider' => [
      'html' => AdminHtmlRouteProvider::class,
      'permissions' => EntityPermissionsRouteProvider::class,
    ],
  ],
  links: [
    'add-form' => '/admin/structure/box-types/add',
    'edit-form' => '/admin/structure/box-types/manage/{box_type}',
    'delete-form' => '/admin/structure/box-types/manage/{box_type}/delete',
    'entity-permissions-form' => '/admin/structure/box-types/manage/{box_type}/permissions',
    'collection' => '/admin/structure/box-types',
  ],
  admin_permission: 'administer box types',
  bundle_of: 'box',
  label_count: [
    'singular' => '@count box type',
    'plural' => '@count box types',
  ],
  config_export: [
    'label',
    'id',
    'description',
    'new_revision',
    'require_revision_log',
  ]
)]
class BoxType extends ConfigEntityBundleBase implements BoxTypeInterface {

  /**
   * The Box type ID.
   *
   * @var string
   */
  protected string $id;

  /**
   * The Box type label.
   *
   * @var string
   */
  protected string $label;

  /**
   * A brief description of this Box type.
   *
   * @var string
   */
  protected string $description;

  /**
   * Default value of the 'Create new revision' checkbox of this box type.
   *
   * @var bool
   */
  protected bool $new_revision = TRUE;

  /**
   * Default value of the 'Require log message' checkbox of this box type.
   *
   * @var bool
   */
  protected bool $require_revision_log = FALSE;

  /**
   * {@inheritdoc}
   */
  public function isLocked(): bool {
    $locked = \Drupal::state()->get('box.type.locked');
    return !empty($locked[$this->id()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription(): ?string {
    return $this->description ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function id(): ?string {
    return $this->id ?? NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setNewRevision(bool $new_revision): void {
    $this->new_revision = $new_revision;
  }

  /**
   * {@inheritdoc}
   */
  public function shouldCreateNewRevision(): bool {
    return $this->new_revision;
  }

  /**
   * {@inheritdoc}
   */
  public function isRevisionLogRequired(): bool {
    return $this->require_revision_log;
  }

  /**
   * {@inheritdoc}
   */
  public function setRevisionLogRequired(): void {
    $this->require_revision_log = TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function setRevisionLogOptional(): void {
    $this->require_revision_log = FALSE;
  }

}
